@@ -14,17 +14,17 @@ vsf_ascii_ascii_to_bin(const char* p_in, char* p_out, unsigned int in_len)
    * For simplicity, I'm cheating and just ripping out all \r. If someone
    * complains about it breaking something, it'll get fixed.
    */
-  unsigned int index = 0;
+  unsigned int indexx = 0;
   unsigned int written = 0;
-  while (index < in_len)
+  while (indexx < in_len)
   {
-    char the_char = p_in[index];
+    char the_char = p_in[indexx];
     if (the_char != '\r')
     {
       *p_out++ = the_char;
       written++;
     }
-    index++;
+    indexx++;
   }
   return written;
 }
@@ -35,11 +35,11 @@ vsf_ascii_bin_to_ascii(const char* p_in, char* p_out, unsigned int in_len)
   /* Task: translate all \n into \r\n. Note that \r\n becomes \r\r\n. That's
    * what wu-ftpd does, and it's easier :-)
    */
-  unsigned int index = 0;
+  unsigned int indexx = 0;
   unsigned int written = 0;
-  while (index < in_len)
+  while (indexx < in_len)
   {
-    char the_char = p_in[index];
+    char the_char = p_in[indexx];
     if (the_char == '\n')
     {
       *p_out++ = '\r';
@@ -47,7 +47,7 @@ vsf_ascii_bin_to_ascii(const char* p_in, char* p_out, unsigned int in_len)
     }
     *p_out++ = the_char;
     written++;
-    index++;
+    indexx++;
   }
   return written;
 }
