@@ -20,6 +20,7 @@
 #include "str.h"
 #include "tunables.h"
 #include "utility.h"
+#include "sysstr.h"
 #include "sysdeputil.h"
 
 void
@@ -36,10 +37,26 @@ vsf_one_process_start(struct vsf_session* p_sess)
   }
   {
     struct mystr user_name = INIT_MYSTR;
+    struct mystr chdir_str = INIT_MYSTR;
     str_alloc_text(&user_name, tunable_ftp_username);
-    vsf_secutil_change_credentials(&user_name, 0, caps,
-      VSF_SECUTIL_OPTION_CHROOT | VSF_SECUTIL_OPTION_USE_GROUPS);
+    if (tunable_anon_root)
+    {
+      str_alloc_text(&chdir_str, tunable_anon_root);
+    }
+    if (tunable_run_as_launching_user)
+    {
+      if (!str_isempty(&chdir_str))
+      {
+        str_chdir(&chdir_str);
+      }
+    }
+    else
+    {
+      vsf_secutil_change_credentials(&user_name, 0, &chdir_str, caps,
+          VSF_SECUTIL_OPTION_CHROOT | VSF_SECUTIL_OPTION_USE_GROUPS);
+    }
     str_free(&user_name);
+    str_free(&chdir_str);
   }
   init_connection(p_sess);
 }
