@@ -203,6 +203,7 @@ struct vsf_sysutil_socketpair_retval
   int socket_one;
   int socket_two;
 };
+void vsf_sysutil_sockaddr_alloc(struct vsf_sysutil_sockaddr** p_sockptr);
 void vsf_sysutil_sockaddr_clear(struct vsf_sysutil_sockaddr** p_sockptr);
 void vsf_sysutil_sockaddr_alloc_ipv4(struct vsf_sysutil_sockaddr** p_sockptr);
 void vsf_sysutil_sockaddr_alloc_ipv6(struct vsf_sysutil_sockaddr** p_sockptr);
@@ -238,7 +239,7 @@ int vsf_sysutil_bind(int fd, const struct vsf_sysutil_sockaddr* p_sockptr);
 void vsf_sysutil_listen(int fd, const unsigned int backlog);
 void vsf_sysutil_getsockname(int fd, struct vsf_sysutil_sockaddr** p_sockptr);
 void vsf_sysutil_getpeername(int fd, struct vsf_sysutil_sockaddr** p_sockptr);
-int vsf_sysutil_accept_timeout(int fd, struct vsf_sysutil_sockaddr** p_sockptr,
+int vsf_sysutil_accept_timeout(int fd, struct vsf_sysutil_sockaddr* p_sockaddr,
                                unsigned int wait_seconds);
 int vsf_sysutil_connect_timeout(int fd,
                                 const struct vsf_sysutil_sockaddr* p_sockaddr,
