@@ -22,6 +22,7 @@
 #include "utility.h"
 #include "tunables.h"
 #include "defs.h"
+#include "parseconf.h"
 
 static void drop_all_privs(void);
 static void handle_sigchld(int duff);
@@ -29,13 +30,16 @@ static void process_login_req(struct vsf_session* p_sess);
 static void common_do_login(struct vsf_session* p_sess,
                             const struct mystr* p_user_str, int do_chroot,
                             int anon);
+static void handle_per_user_config(const struct mystr* p_user_str);
+static void calculate_chdir_dir(int anon, struct mystr* p_userdir_str,
+                                struct mystr* p_chroot_str,
+                                struct mystr* p_chdir_str,
+                                const struct mystr* p_user_str,
+                                const struct mystr* p_orig_user_str);
 
 static void
 handle_sigchld(int duff)
 {
-  /* WARNING - async handler. Must not call anything which might have
-   * re-entrancy issues
-   */
   struct vsf_sysutil_wait_retval wait_retval = vsf_sysutil_wait();
   (void) duff;
   /* Child died, so we'll do the same! Report it as an error unless the child
@@ -68,8 +72,6 @@ vsf_two_process_start(struct vsf_session* p_sess)
       {
         process_login_req(p_sess);
       }
-      /* NOTREACHED */
-      bug("should not get here: vsf_two_process_start");
     }
   }
   /* Child process - time to lose as much privilege as possible and do the
@@ -81,7 +83,7 @@ vsf_two_process_start(struct vsf_session* p_sess)
                               VSFTP_CONF_FILE_MAX);
     if (vsf_sysutil_retval_is_error(retval))
     {
-      die("cannot open user list file");
+      die2("cannot open user list file:", tunable_userlist_file);
     }
   }
   drop_all_privs();
@@ -101,11 +103,12 @@ drop_all_privs(void)
     struct vsf_sysutil_statbuf* p_statbuf = 0;
     if (vsf_sysutil_retval_is_error(str_lstat(&dir_str, &p_statbuf)))
     {
-      die("vsftpd: not found: directory given in 'tunable_secure_chroot_dir'");
+      die2("vsftpd: not found: directory given in 'secure_chroot_dir':",
+           tunable_secure_chroot_dir);
     }
     vsf_sysutil_free(p_statbuf);
   }
-  vsf_secutil_change_credentials(&user_str, &dir_str, 0,
+  vsf_secutil_change_credentials(&user_str, &dir_str, 0, 0,
                                  VSF_SECUTIL_OPTION_CHROOT);
   str_free(&user_str);
   str_free(&dir_str);
@@ -169,8 +172,12 @@ static void
 process_login_req(struct vsf_session* p_sess)
 {
   enum EVSFPrivopLoginResult e_login_result = kVSFLoginNull;
+  char cmd;
+  vsf_sysutil_unblock_sig(kVSFSysUtilSigCHLD);
   /* Blocks */
-  if (priv_sock_get_cmd(p_sess) != PRIV_SOCK_LOGIN)
+  cmd = priv_sock_get_cmd(p_sess);
+  vsf_sysutil_block_sig(kVSFSysUtilSigCHLD);
+  if (cmd != PRIV_SOCK_LOGIN)
   {
     die("bad request");
   }
@@ -207,7 +214,8 @@ process_login_req(struct vsf_session* p_sess)
                                     VSFTP_CONF_FILE_MAX);
           if (vsf_sysutil_retval_is_error(retval))
           {
-            die("cannot open chroot() user list file");
+            die2("could not open chroot() list file:",
+                 tunable_chroot_list_file);
           }
           if (str_contains_line(&chroot_list_file, &p_sess->user_str))
           {
@@ -236,42 +244,64 @@ static void
 common_do_login(struct vsf_session* p_sess, const struct mystr* p_user_str,
                 int do_chroot, int anon)
 {
+  int was_anon = anon;
+  const struct mystr* p_orig_user_str = p_user_str;
   int newpid;
-  vsf_sysutil_default_sig(kVSFSysUtilSigCHLD);
+  vsf_sysutil_install_null_sighandler(kVSFSysUtilSigCHLD);
   /* Asks the pre-login child to go away (by exiting) */
   priv_sock_send_result(p_sess, PRIV_SOCK_RESULT_OK);
   (void) vsf_sysutil_wait();
+  /* Absorb the SIGCHLD */
+  vsf_sysutil_unblock_sig(kVSFSysUtilSigCHLD);
+  /* Handle loading per-user config options */
+  handle_per_user_config(p_user_str);
+  /* Set this before we fork */
+  p_sess->is_anonymous = anon;
   vsf_sysutil_install_async_sighandler(kVSFSysUtilSigCHLD, handle_sigchld);
   newpid = vsf_sysutil_fork(); 
   if (newpid == 0)
   {
     struct mystr guest_user_str = INIT_MYSTR;
+    struct mystr chroot_str = INIT_MYSTR;
+    struct mystr chdir_str = INIT_MYSTR;
+    struct mystr userdir_str = INIT_MYSTR;
     unsigned int secutil_option = VSF_SECUTIL_OPTION_USE_GROUPS;
-    if (do_chroot)
-    {
-      secutil_option |= VSF_SECUTIL_OPTION_CHROOT;
-    }
     /* Child - drop privs and start proper FTP! */
     if (tunable_guest_enable && !anon)
     {
       /* Remap to the guest user */
       str_alloc_text(&guest_user_str, tunable_guest_username);
       p_user_str = &guest_user_str;
-      /* SECURITY: For now, apply the anonymous restrictions to
-       * guest users
-       */
-      anon = 1;
+      if (!tunable_virtual_use_local_privs)
+      {
+        anon = 1;
+        do_chroot = 1;
+      }
+    }
+    if (do_chroot)
+    {
+      secutil_option |= VSF_SECUTIL_OPTION_CHROOT;
     }
     if (!anon)
     {
       secutil_option |= VSF_SECUTIL_OPTION_CHANGE_EUID;
     }
-    vsf_secutil_change_credentials(p_user_str, 0, 0, secutil_option);
+    calculate_chdir_dir(was_anon, &userdir_str, &chroot_str, &chdir_str,
+                        p_user_str, p_orig_user_str);
+    vsf_secutil_change_credentials(p_user_str, &userdir_str, &chroot_str,
+                                   0, secutil_option);
+    if (!str_isempty(&chdir_str))
+    {
+      (void) str_chdir(&chdir_str);
+    }
     str_free(&guest_user_str);
+    str_free(&chroot_str);
+    str_free(&chdir_str);
+    str_free(&userdir_str);
     /* Guard against the config error of having the anonymous ftp tree owned
      * by the user we are running as
      */
-    if (anon && vsf_sysutil_write_access("/"))
+    if (was_anon && vsf_sysutil_write_access("/"))
     {
       die("vsftpd: refusing to run with writable anonymous root");
     }
@@ -284,3 +314,83 @@ common_do_login(struct vsf_session* p_sess, const struct mystr* p_user_str,
   bug("should not get here in common_do_login");
 }
 
+static void
+handle_per_user_config(const struct mystr* p_user_str)
+{
+  struct mystr filename_str = INIT_MYSTR;
+  struct vsf_sysutil_statbuf* p_statbuf = 0;
+  struct str_locate_result loc_result;
+  int retval;
+  if (!tunable_user_config_dir)
+  {
+    return;
+  }
+  /* Security paranoia - ignore if user has a / in it. */
+  loc_result = str_locate_char(p_user_str, '/');
+  if (loc_result.found)
+  {
+    return;
+  }
+  str_alloc_text(&filename_str, tunable_user_config_dir);
+  str_append_char(&filename_str, '/');
+  str_append_str(&filename_str, p_user_str);
+  retval = str_stat(&filename_str, &p_statbuf);
+  /* Security - ignore unless owned by root */
+  if (!vsf_sysutil_retval_is_error(retval) &&
+      vsf_sysutil_statbuf_get_uid(p_statbuf) == VSFTP_ROOT_UID)
+  {
+    vsf_parseconf_load_file(str_getbuf(&filename_str), 1);
+  }
+  str_free(&filename_str);
+  vsf_sysutil_free(p_statbuf);
+}
+
+static void
+calculate_chdir_dir(int anon_login, struct mystr* p_userdir_str,
+                    struct mystr* p_chroot_str,
+                    struct mystr* p_chdir_str,
+                    const struct mystr* p_user_str,
+                    const struct mystr* p_orig_user_str)
+{
+  if (!anon_login)
+  {
+    const struct vsf_sysutil_user* p_user = str_getpwnam(p_user_str);
+    if (p_user == 0)
+    {
+      die2("cannot locate user entry:", str_getbuf(p_user_str));
+    }
+    str_alloc_text(p_userdir_str, vsf_sysutil_user_get_homedir(p_user));
+    if (tunable_user_sub_token)
+    {
+      str_replace_text(p_userdir_str, tunable_user_sub_token,
+                       str_getbuf(p_orig_user_str));
+    }
+  }
+  if (anon_login && tunable_anon_root)
+  {
+    str_alloc_text(p_chroot_str, tunable_anon_root);
+  }
+  else if (!anon_login && tunable_local_root)
+  {
+    str_alloc_text(p_chroot_str, tunable_local_root);
+    if (tunable_user_sub_token)
+    {
+      str_replace_text(p_chroot_str, tunable_user_sub_token,
+                       str_getbuf(p_orig_user_str));
+    }
+  }
+  /* If enabled, the chroot() location embedded in the HOMEDIR takes
+   * precedence.
+   */
+  if (!anon_login && tunable_passwd_chroot_enable)
+  {
+    struct str_locate_result loc_result;
+    loc_result = str_locate_text(p_userdir_str, "/./");
+    if (loc_result.found)
+    {
+      str_split_text(p_userdir_str, p_chdir_str, "/./");
+      str_copy(p_chroot_str, p_userdir_str);
+    }
+  }
+}
+
